/*
 *  Copyright (C) 2005 SAN People
 *  Copyright (C) 2008 Atmel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/types.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/spi/spi.h>
#include <linux/spi/at73c213.h>
#include <linux/clk.h>

#include <mach/hardware.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/irq.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <mach/board.h>
#include <mach/gpio.h>
#include <mach/at91sam9_smc.h>

#include "sam9_smc.h"
#include "generic.h"


static void __init matrix504_map_io(void)
{
	/* Initialize processor: 18.432 MHz crystal */
	at91sam9260_initialize(18432000);

	/* DGBU on ttyS0. (Rx & Tx only) */
	at91_register_uart(0, 0, 0);

	/* USART0 on ttyS1. (Rx, Tx, RTS, CTS) */
	at91_register_uart(AT91SAM9260_ID_US0, 1, ATMEL_UART_CTS | ATMEL_UART_RTS 
			   | ATMEL_UART_DTR | ATMEL_UART_DSR | ATMEL_UART_DCD);

	/* USART1 on ttyS2. (Rx, Tx, CTS, RTS, DTR, DSR, DCD, RI) */
	at91_register_uart(AT91SAM9260_ID_US1, 2, ATMEL_UART_CTS | ATMEL_UART_RTS);
	/* USART2 on ttyS3. (Rx, Tx, RTS, CTS) */
	at91_register_uart(AT91SAM9260_ID_US2, 3, ATMEL_UART_CTS | ATMEL_UART_RTS);
	/* USART3 on ttyS4. (Rx, Tx, RTS, CTS) */
	at91_register_uart(AT91SAM9260_ID_US3, 4, ATMEL_UART_CTS | ATMEL_UART_RTS);
	/* set serial console to ttyS0 (ie, DBGU) */
	at91_set_serial_console(0);
}

static void __init matrix504_init_irq(void)
{
	at91sam9260_init_interrupts(NULL);
}


/*
 * USB Host port
 */
static struct at91_usbh_data __initdata matrix504_usbh_data = {
	.ports		= 2,
};

/*
 * USB Device port
 */
static struct at91_udc_data __initdata matrix504_udc_data = {
	.vbus_pin	= AT91_PIN_PC5,
	.pullup_pin	= 0,		/* pull-up driven by UDC */
};


/*
 * SPI devices.!defined(CONFIG_MMC_AT91)
 */
static struct spi_board_info matrix504_spi_devices[] = {
	{	/* DataFlash chip */
		.modalias	= "mtd_dataflash",
		.chip_select	= 1,
		.max_speed_hz	= 15 * 1000 * 1000,
		.bus_num	= 0,
	}
};


/*
 * MACB Ethernet device
 */
static struct at91_eth_data __initdata matrix504_macb_data_V03 = {
	.phy_irq_pin	= AT91_PIN_PA7,
	.is_rmii	= 1,
};

static struct at91_eth_data __initdata matrix504_macb_data_V04 = {
	.phy_irq_pin	= AT91_PIN_PB20,
	.is_rmii	= 1,
};


/* det_pin is not connected */
static struct atmel_nand_data __initdata matrix504_nand_data = {
	.ale		= 21,
	.cle		= 22,
	.rdy_pin	= AT91_PIN_PC13,
	.enable_pin	= AT91_PIN_PC14,
	//.partition_info	= nand_partitions,
#if defined(CONFIG_MTD_NAND_ATMEL_BUSWIDTH_16)
	.bus_width_16	= 1,
#else
	.bus_width_16	= 0,
#endif
};

static struct sam9_smc_config __initdata matrix504_nand_smc_config = {
	.ncs_read_setup		= 0,
	.nrd_setup		= 2,
	.ncs_write_setup	= 0,
	.nwe_setup		= 2,

	.ncs_read_pulse		= 4,
	.nrd_pulse		= 4,
	.ncs_write_pulse	= 4,
	.nwe_pulse		= 4,

	.read_cycle		= 7,
	.write_cycle		= 7,

	.mode			= AT91_SMC_READMODE | AT91_SMC_WRITEMODE | AT91_SMC_EXNWMODE_DISABLE,
	.tdf_cycles		= 3,
};

static void __init matrix504_add_device_nand(void)
{
	/* setup bus-width (8 or 16) */
	if (matrix504_nand_data.bus_width_16)
		matrix504_nand_smc_config.mode |= AT91_SMC_DBW_16;
	else
		matrix504_nand_smc_config.mode |= AT91_SMC_DBW_8;

	/* configure chip-select 3 (NAND) */
	sam9_smc_configure(3, &matrix504_nand_smc_config);

	at91_add_device_nand(&matrix504_nand_data);
}

/*
 * MCI (SD/MMC)
 * det_pin, wp_pin and vcc_pin are not connected
 */
static struct at91_mmc_data __initdata m502_mmc_data = {
	.slot_b		= 0,
	.wire4		= 1,
	.det_pin	= AT91_PIN_PA3,
};

/*
 * LEDs
 */
static struct gpio_led matrix504_leds[] = {
	{	
		.name			= "led0",
		.gpio			= AT91_PIN_PA22,
#ifdef CONFIG_MINI_MATRIX
		.active_low		= 0,
#else
		.active_low		= 1,
#endif
		.default_trigger	= "none",
	},
	{	
		.name			= "led1",
		.gpio			= AT91_PIN_PA25,
		.active_low		= 1,
		.default_trigger	= "none",
	},
	{	
		.name			= "led2",
		.gpio			= AT91_PIN_PA27,
		.active_low		= 1,
		.default_trigger	= "none",
	},
	{	
		.name			= "led3",
		.gpio			= AT91_PIN_PA28,
		.active_low		= 1,
		.default_trigger	= "none",
	},
};


static struct i2c_board_info __initdata matrix504_i2c_devices[] = {
	{
		I2C_BOARD_INFO("m41t81", 0x68),
	}
};

static int check_m504_version(void) {

	at91_set_gpio_input(AT91_PIN_PB18,1);
	at91_set_gpio_input(AT91_PIN_PB19,1);
	if(at91_get_gpio_value(AT91_PIN_PB18) == 0 && at91_get_gpio_value(AT91_PIN_PB19) == 0)
		return 1; //V04
	
	return 0;//V03
}

static void __init matrix504_board_init(void)
{
	/* Serial */
	at91_add_device_serial();
	/* USB Host */
	at91_add_device_usbh(&matrix504_usbh_data);
	/* USB Device */
	at91_add_device_udc(&matrix504_udc_data);
	/* SPI */
	//Artila Ace set 
	at91_add_device_spi(matrix504_spi_devices, ARRAY_SIZE(matrix504_spi_devices));//0);//
	/* NAND */
	matrix504_add_device_nand();
	if(check_m504_version() == 1) { //V04
		/* Ethernet */
		at91_add_device_eth(&matrix504_macb_data_V04);
		/* MMC */
		at91_add_device_mmc(0, &m502_mmc_data);
		printk("Matrix504 Version 04\n");
	}else{
		/* Ethernet */
		at91_add_device_eth(&matrix504_macb_data_V03);
		printk("Matrix504 Version 03\n");
	}

	/* I2C */
	at91_add_device_i2c(matrix504_i2c_devices, ARRAY_SIZE(matrix504_i2c_devices));
	/* LEDs */
	at91_gpio_leds(matrix504_leds, 4);
}

MACHINE_START(MATRIX504, "Matrix504")
	/* Maintainer: Atmel */
	.phys_io	= AT91_BASE_SYS,
	.io_pg_offst	= (AT91_VA_BASE_SYS >> 18) & 0xfffc,
	.boot_params	= AT91_SDRAM_BASE + 0x100,
	.timer		= &at91sam926x_timer,
	.map_io		= matrix504_map_io,
	.init_irq	= matrix504_init_irq,
	.init_machine	= matrix504_board_init,
MACHINE_END
