#include <linux/kd.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <asm/bitops.h> 
#include <asm/uaccess.h>
#include <linux/init.h>
#include <linux/spinlock.h>
#include <mach/hardware.h>
#include <mach/at91_pio.h>
#include <mach/gpio.h>
#include <mach/timex.h>
#include <linux/delay.h>

#define GPIO_MINOR 		253

static DEFINE_SPINLOCK(buzzer_lock);
static int buzzer_open_cnt=0;

static int matrix504_buzzer_open(struct inode *inode, struct file *file)
{
	spin_lock(&buzzer_lock);
	buzzer_open_cnt++;
	spin_unlock(&buzzer_lock);
	return 0;
}


static int matrix504_buzzer_close(struct inode *inode, struct file *file)
{
	spin_lock(&buzzer_lock);
	spin_unlock(&buzzer_lock);
	return 0;
}

/*
 * Handle commands from user-space.
 */
static int matrix504_buzzer_ioctl(struct inode *inode, struct file *file,
		unsigned int cmd, unsigned long arg)
{

	unsigned int ticks, count, i,j;

	switch(cmd){
		case KDMKTONE:
			
			/*
			* Generate the tone for the appropriate number of ticks.
			* If the time is zero, turn off sound ourselves.
			*/
			ticks = HZ * ((arg >> 16) & 0xffff) / 1000;
			count = ticks ? (arg & 0xffff) : 0;
			
			count = (count/23);
			
			for(i=0; i<= ticks;i++){
				at91_set_gpio_value(AT91_PIN_PA26, 1);
				
				if(count > 1000) {
					j = (count / 1000);
					mdelay(j);
					udelay(count - j*1000);
				}else
					udelay(count);

				at91_set_gpio_value(AT91_PIN_PA26, 0);
	
				if(count > 1000) {
					j = (count / 1000);
					mdelay(j);
					udelay(count - j*1000);
				}else
					udelay(count);
						
			}
		break;
	}
	return -ENOIOCTLCMD;
}

/* ......................................................................... */

static struct file_operations matrix504buzzer_fops =
{
	.owner		= THIS_MODULE,
	.ioctl		= matrix504_buzzer_ioctl,
	.open		= matrix504_buzzer_open,
	.release	= matrix504_buzzer_close,
};

static struct miscdevice matrix504_buzzer_miscdev =
{
	.minor		= GPIO_MINOR,
	.name		= "buzzer",
	.fops		= &matrix504buzzer_fops,	
};

static int __init matrix504_buzzer_init(void)
{
	int res;
	//int i;

	res = misc_register(&matrix504_buzzer_miscdev);
	if (res)
		return res;

	at91_set_gpio_output(AT91_PIN_PA26, 1);
	printk("M504 Buzzer Driver Ver 1.00 Loaded.\n");
	return 0;
}

static void __exit matrix504_buzzer_exit(void)
{
	misc_deregister(&matrix504_buzzer_miscdev);
	
}

module_init(matrix504_buzzer_init);
module_exit(matrix504_buzzer_exit);

MODULE_LICENSE("GPL")
MODULE_AUTHOR("Ace Yang")
MODULE_DESCRIPTION("Buzzer driver for M504 based on AT91SAM9G20")