/*
 *This Driver only for Artila . 
 *	By Ace Yang
 */


#include <linux/kernel.h>
#include <linux/platform_device.h>
#include <linux/leds.h>
#include <linux/io.h>
#include <linux/proc_fs.h> 
#include <asm/uaccess.h> 
#include <linux/ioport.h> 
#include <linux/module.h>
#include <linux/init.h>

#include <mach/board.h>
#include <mach/gpio.h>

static struct proc_dir_entry *led_entry;
static int LEDswitch[5]={0,0,0,0,0};

static int matrix504_ready_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data);
static int matrix504_led1_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data);
static int matrix504_led2_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data);
static int matrix504_led3_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data);
static int matrix504_led4_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data);
static int matrix504_ready_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data);
static int matrix504_led1_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data);
static int matrix504_led2_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data);
static int matrix504_led3_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data);
static int matrix504_led4_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data);

static struct matrix504_proc_list {
	const char *name;	/**< file name */
	int (*f) (char *, char **, off_t, int, int *, void *);		/**< proc callback*/
	int (*w) (struct file *, const char __user *,unsigned long , void *);
} matrix504_proc_list[] = {
	{"ready", matrix504_ready_read,matrix504_ready_write},
	{"led1", matrix504_led1_read,matrix504_led1_write},
	{"led2", matrix504_led2_read,matrix504_led2_write},
	{"led3", matrix504_led3_read,matrix504_led3_write},
	{"led4", matrix504_led4_read,matrix504_led4_write},
};

static int led_pin[5] ={
	AT91_PIN_PC6,
	AT91_PIN_PA22,
	AT91_PIN_PA25,
	AT91_PIN_PA27,
	AT91_PIN_PA28,
};

#define MATRIX504_PROC_ENTRIES ARRAY_SIZE(matrix504_proc_list)

/*
 * Artila Ace : read proc status
 */
static int matrix504_status_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data, int flag)
{
	char *p = resp_buff;
	int len;
  
	p += sprintf(p, "%d\n", LEDswitch[flag]);
	len = (p - resp_buff) - offset;
	if (len < 0) 
		len = 0;
	*eof = (len <= count) ? 1 : 0;
	*start = resp_buff + offset;
	return len;
}

static int matrix504_ready_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data) 
{
	return matrix504_status_read(resp_buff,start,offset,count,eof,data,0);
}

static int matrix504_led1_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data) 
{
	return matrix504_status_read(resp_buff,start,offset,count,eof,data,1);
}

static int matrix504_led2_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data) 
{
	return matrix504_status_read(resp_buff,start,offset,count,eof,data,2);
}

static int matrix504_led3_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data) 
{
	return matrix504_status_read(resp_buff,start,offset,count,eof,data,3);
}

static int matrix504_led4_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data) 
{
	return matrix504_status_read(resp_buff,start,offset,count,eof,data,4);
}

/*
 * Artila Ace : write proc
 */

static int matrix504_status_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data,int flag)
{
	char mode;
	if (count > 0) {
		if (get_user(mode, buffer)) 
			return -EFAULT;
		if(mode == '1') {
			at91_set_gpio_output(led_pin[flag],0);
			LEDswitch[flag] = 1;
		}
		if(mode == '0')	{
			at91_set_gpio_output(led_pin[flag],1);
			LEDswitch[flag] = 0;
		}
	}
	return count;
}
static int matrix504_ready_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data)
{
	return matrix504_status_write(file,buffer,count,data,0);
}
static int matrix504_led1_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data)
{
	return matrix504_status_write(file,buffer,count,data,1);
}
static int matrix504_led2_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data)
{
	return matrix504_status_write(file,buffer,count,data,2);
}
static int matrix504_led3_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data)
{
	return matrix504_status_write(file,buffer,count,data,3);
}
static int matrix504_led4_write(struct file *file, const char __user *buffer,
			       unsigned long count, void *data)
{
	return matrix504_status_write(file,buffer,count,data,4);
}

static int __init matrix504_init(void) 
{ 
	struct proc_dir_entry *ent;
	int i, j, ret;
	
	/* Set up the proc file system */
	led_entry = proc_mkdir("led", NULL);
	if (!led_entry) {
		printk("Can't create /proc/led\n");
		return  -ENOMEM;
	}
	
	led_entry->owner=THIS_MODULE;
	
	for(i = 0 ; i < MATRIX504_PROC_ENTRIES ; i++) {
		ent = create_proc_entry(matrix504_proc_list[i].name, 0644, led_entry);
		if (!ent) {
			printk("Error : LED Driver\n");
			ret = -ENOMEM;
			goto fail;
		}
		ent->data = NULL;
		ent->read_proc = matrix504_proc_list[i].f;
		ent->write_proc = matrix504_proc_list[i].w;
		ent->owner=THIS_MODULE;
	}
	
	printk("Matrix504 LED Driver\n");
	return 0;
fail:
	for (j = 0; j < i; j++)
		remove_proc_entry(matrix504_proc_list[i].name,
				  led_entry);
	remove_proc_entry("led", led_entry);
	led_entry = NULL;
	return ret;
} 
  
static void __exit matrix504_exit(void) 
{ 
	int i ;
	for(i = 0 ; i < MATRIX504_PROC_ENTRIES ; i++) 
		remove_proc_entry(matrix504_proc_list[i].name,
				  led_entry);
			  
	remove_proc_entry("led", led_entry);
} 

module_init(matrix504_init);
module_exit(matrix504_exit);

MODULE_DESCRIPTION("Artila led module");
MODULE_AUTHOR("Ace Yang");
MODULE_LICENSE("GPL");
MODULE_ALIAS("Artila Matrix504:LED module");